@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2003 Rochet Sylvain
+ *  Copyright (C) 2002-2004 Rochet Sylvain
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -86,7 +86,11 @@ void read_config_files()   {
 
 	/* read default config file */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/default.conf", NULL);
-	if(! parse_file_config(filename) ) exit(-1);
+	if(! parse_file_config(filename) ) {
+		printf("Error opening config file %s\n",filename);
+		exit(-1);
+	}
+
 	g_free(filename);
 
 	/* read user config file */
@@ -161,25 +165,32 @@ void create_connection_get_games(guchar *host, guint32 port)  {
 
 
 /* insert test in Message Box */
-void text_insert_message(gchar* text, guint32 lenght)  {
+void text_insert_message(gchar* text, guint32 length)  {
+
+	GtkTextBuffer *textbuff;
+	GtkTextIter textiter;
+	GtkTextMark *textmark;
 
 	guint32 i, len;
 
 	if(global->phase < PHASE_GETGAMES) return;
 
+	textbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(global->Message));
+	gtk_text_buffer_get_end_iter(textbuff, &textiter);
+	gtk_text_buffer_insert(textbuff, &textiter, text, length);
+	gtk_text_buffer_insert(textbuff, &textiter, "\n", -1);
 
-	gtk_text_freeze( GTK_TEXT(global->Message) );
+	/* Scroll the text */
+	textmark = gtk_text_buffer_get_mark(textbuff, "endmark");
+	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(global->Message), textmark,
+			0.0, FALSE, 0.0, 0.0);
 
-	len = gtk_text_get_length( GTK_TEXT(global->Message) );
-	gtk_text_set_point( GTK_TEXT(global->Message), len);
-	if(len > 0)  gtk_text_insert(GTK_TEXT(global->Message), NULL, NULL, NULL, "\n", 1);
-	gtk_text_insert(GTK_TEXT(global->Message), NULL, NULL, NULL, text, lenght);
 	global->message_nb_lines++;
 
-	while(global->message_nb_lines > config->message_max_lines) {
+//	while(global->message_nb_lines > config->message_max_lines) {
 
 		/* if unlimited */
-		if(config->message_max_lines <= 0)  break;
+/*		if(config->message_max_lines <= 0)  break;
 
 		len = gtk_text_get_length( GTK_TEXT(global->Message) );
 
@@ -187,7 +198,7 @@ void text_insert_message(gchar* text, guint32 lenght)  {
 
 			if(i >= len)  { /* argl... no CR founded */
 
-				gtk_text_set_point( GTK_TEXT(global->Message), 0);
+/*				gtk_text_set_point( GTK_TEXT(global->Message), 0);
 				gtk_text_forward_delete( GTK_TEXT(global->Message), len);
 				global->message_nb_lines = 0;
 				break;
@@ -209,28 +220,38 @@ void text_insert_message(gchar* text, guint32 lenght)  {
 	gtk_text_thaw( GTK_TEXT(global->Message) );
 
 	gtk_adjustment_set_value( GTK_ADJUSTMENT( GTK_TEXT( global->Message )->vadj ),  GTK_ADJUSTMENT( GTK_TEXT( global->Message )->vadj )->upper );
+	*/
 }
 
 
-/* insert test in Chat Box */ 
-void text_insert_chat(gchar* text, guint32 lenght)  {
+/* insert test in Chat Box */
+void text_insert_chat(gchar* text, guint32 length)
+{
+	GtkTextBuffer *textbuff;
+	GtkTextIter textiter;
+	GtkTextMark *textmark;
 
 	guint32 i, len;
 
-	if(global->phase < PHASE_GAMECREATE) return;
+	if(global->phase < PHASE_GAMECREATE)
+		return;
 
-	gtk_text_freeze( GTK_TEXT(game->Chat) );
+	textbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(game->Chat));
+	gtk_text_buffer_get_end_iter(textbuff, &textiter);
+	gtk_text_buffer_insert(textbuff, &textiter, text, length);
+	gtk_text_buffer_insert(textbuff, &textiter, "\n", -1);
+
+	/* Scroll to the end mark */
+	textmark = gtk_text_buffer_get_mark(textbuff, "endmark");
+	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(game->Chat), textmark,
+			0.0, FALSE, 0.0, 0.0);
 
-	len = gtk_text_get_length( GTK_TEXT(game->Chat) );
-	gtk_text_set_point( GTK_TEXT(game->Chat), len);
-	if(len > 0)  gtk_text_insert(GTK_TEXT(game->Chat), NULL, NULL, NULL, "\n", 1);
-	gtk_text_insert(GTK_TEXT(game->Chat), NULL, NULL, NULL, text, lenght);
 	game->chat_nb_lines++;
 
-	while(game->chat_nb_lines > config->chat_max_lines) {
+/*	while(game->chat_nb_lines > config->chat_max_lines) {
 
 		/* if unlimited */
-		if(config->chat_max_lines <= 0)  break;
+/*		if(config->chat_max_lines <= 0)  break;
 
 		len = gtk_text_get_length( GTK_TEXT(game->Chat) );
 
@@ -238,7 +259,7 @@ void text_insert_chat(gchar* text, guint32 lenght)  {
 
 			if(i >= len)  { /* argl... no CR founded */
 
-				gtk_text_set_point( GTK_TEXT(game->Chat), 0);
+/*				gtk_text_set_point( GTK_TEXT(game->Chat), 0);
 				gtk_text_forward_delete( GTK_TEXT(game->Chat), len);
 				game->chat_nb_lines = 0;
 				break;
@@ -259,7 +280,9 @@ void text_insert_chat(gchar* text, guint32 lenght)  {
 
 	gtk_text_thaw( GTK_TEXT(game->Chat) );
 
-	gtk_adjustment_set_value( GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj ),  GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj )->upper );
+//	gtk_adjustment_set_value( GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj ),  GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj )->upper );
+
+	*/
 }
 
 
@@ -287,14 +310,14 @@ void game_free()  {
 
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(! game->player[i].playerid) continue;
+		if(! global->player[i].playerid) continue;
 
-		if(game->player[i].name) g_free(game->player[i].name);
-		if(game->player[i].host) g_free(game->player[i].host);
-		if(game->player[i].image) g_free(game->player[i].image);
+		if(global->player[i].name) g_free(global->player[i].name);
+		if(global->player[i].host) g_free(global->player[i].host);
+		if(global->player[i].image) g_free(global->player[i].image);
 
-		frame_destroy( game->player[i].playerlist_token_frame );
-		frame_destroy( game->player[i].playerlist_cards_frame );
+		frame_destroy( global->player[i].playerlist_token_frame );
+		frame_destroy( global->player[i].playerlist_cards_frame );
 	}
 
 	for(i = 0 ; i < data->number_estates ; i++)  {
@@ -330,10 +353,10 @@ void game_free()  {
 /* return a valid game id */
 gboolean game_get_valid_player_slot(guint8 *playerslot)  {
 
-	guint8 i;
+	guint32 i;
 
 	for(i = 0; i < MAX_PLAYERS; i++)
-		if(!game->player[i].playerid) {
+		if(!global->player[i].playerid) {
 
 			*playerslot = i;
 			return TRUE;
@@ -350,7 +373,7 @@ gint8 get_player_slot_with_playerid(guint32 playerid)  {
 
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(game->player[i].playerid == playerid) {
+		if(global->player[i].playerid == playerid) {
 
 			return(i);
 			break;
@@ -438,7 +461,7 @@ void parse_specific_chat_message(guchar *message)   {
 			else  msg++;
 		}
 
-		if(strlen(msg) == 0  ||  !strcmp(msg, game->player[pslot].name) )  {
+		if(strlen(msg) == 0  ||  !strcmp(msg, global->player[pslot].name) )  {
 
 			sendstr = g_strdup_printf("GtkAtlantic %s\n", VERSION);
 			client_send(game->connectid, sendstr, strlen(sendstr));
@@ -456,7 +479,7 @@ void parse_specific_chat_message(guchar *message)   {
 			else  msg++;
 		}
 
-		if(strlen(msg) == 0  ||  !strcmp(msg, game->player[pslot].name) )  {
+		if(strlen(msg) == 0  ||  !strcmp(msg, global->player[pslot].name) )  {
 
 			gettimeofday(&tv, NULL);
 			t_now = tv.tv_sec;
@@ -475,7 +498,7 @@ void parse_specific_chat_message(guchar *message)   {
 			else  msg++;
 		}
 
-		if(strlen(msg) == 0  ||  !strcmp(msg, game->player[pslot].name) )  {
+		if(strlen(msg) == 0  ||  !strcmp(msg, global->player[pslot].name) )  {
 
 			sendstr = g_strdup_printf("pong\n");
 			client_send(game->connectid, sendstr, strlen(sendstr));
@@ -514,9 +537,10 @@ void parse_specific_send_message(guchar *message)   {
 
 			for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-				if(! game->player[i].playerid)  continue;
+				if(! global->player[i].playerid)  continue;
+				if(global->player[i].game != game->gameid)  continue;
 
-				tmp = g_strdup_printf("<%s> %d", game->player[i].name, game_get_assets_player(game->player[i].playerid) );
+				tmp = g_strdup_printf("<%s> %d", global->player[i].name, game_get_assets_player(global->player[i].playerid) );
 				text_insert_message(tmp, strlen(tmp) );
 				g_free(tmp);
 			}
@@ -526,7 +550,7 @@ void parse_specific_send_message(guchar *message)   {
 
 			pslot = get_player_slot_with_playerid(playerid);
 
-			tmp = g_strdup_printf("<%s> %d", game->player[pslot].name, game_get_assets_player(game->player[pslot].playerid) );
+			tmp = g_strdup_printf("<%s> %d", global->player[pslot].name, game_get_assets_player(global->player[pslot].playerid) );
 			text_insert_message(tmp, strlen(tmp) );
 			g_free(tmp);
 		}
@@ -542,7 +566,7 @@ void parse_specific_send_message(guchar *message)   {
 			else  msg++;
 		}
 
-		if(strlen(msg) != 0  &&  strcmp(game->player[ get_player_slot_with_playerid(game->my_playerid) ].name, msg) )  {
+		if(strlen(msg) != 0  &&  strcmp(global->player[ get_player_slot_with_playerid(game->my_playerid) ].name, msg) )  {
 
 			tmp = g_strdup_printf(".n%s\n", msg);
 			client_send(game->connectid, tmp, strlen(tmp));
@@ -612,17 +636,17 @@ void game_sort_playerlist_by_playerid()  {
 
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(! game->player[i].playerid) continue;
+		if(! global->player[i].playerid) continue;
 
 		for(j = i+1 ; j < MAX_PLAYERS ; j++)  {
 
-			if(! game->player[j].playerid) continue;
+			if(! global->player[j].playerid) continue;
 
-			if(game->player[j].playerid < game->player[i].playerid)  {
+			if(global->player[j].playerid < global->player[i].playerid)  {
 
-				memcpy(&playertmp, &game->player[i], sizeof(_player) );
-				memcpy(&game->player[i], &game->player[j], sizeof(_player) );
-				memcpy(&game->player[j], &playertmp, sizeof(_player) );
+				memcpy(&playertmp, &global->player[i], sizeof(_player) );
+				memcpy(&global->player[i], &global->player[j], sizeof(_player) );
+				memcpy(&global->player[j], &playertmp, sizeof(_player) );
 			}
 		}
 	}
@@ -699,11 +723,11 @@ gint32 get_playerid_by_playername(guchar *name)  {
 
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(! game->player[i].playerid)  continue;
+		if(! global->player[i].playerid)  continue;
 
-		if(! strcmp(game->player[i].name, name) )  {
+		if(! strcmp(global->player[i].name, name) )  {
 
-			return( game->player[i].playerid );
+			return( global->player[i].playerid );
 			break;
 		}
 	}
@@ -784,12 +808,12 @@ void game_buildplayerlist()   {
 
 		for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-			if(game->player[i].playerid)  {
+			if(global->player[i].playerid && global->player[i].game == game->gameid)  {
 
-				txt[0] = game->player[i].name;
-				txt[1] = game->player[i].host;
+				txt[0] = global->player[i].name;
+				txt[1] = global->player[i].host;
 				row = gtk_clist_append(GTK_CLIST(game->PlayerCList), txt);
-				gtk_clist_set_selectable(GTK_CLIST(game->PlayerCList), row ,FALSE);			
+				gtk_clist_set_selectable(GTK_CLIST(game->PlayerCList), row ,FALSE);
 			}
 		}
 
@@ -810,7 +834,7 @@ gint32 game_get_assets_player(guint32 playerid)  {
 	if(pslot < 0)  return 0;
 
 	/* money */
-	assets = game->player[pslot].money;
+	assets = global->player[pslot].money;
 
 	/* estates + houses */
 	for(i = 0 ; i < data->number_estates ; i++)  {
@@ -844,10 +868,11 @@ void game_update_tokens()  {
 		/* search number of unjailed tokens on this estate */
 		for(j = 0, nb_tokens = 0 ; j < MAX_PLAYERS ; j++)  {
 
-			if(! game->player[j].playerid)  continue;
-			if(game->player[j].bankrupt)  continue;
-			if(game->player[j].jailed) continue;
-			if(game->player[j].location != i)  continue;
+			if(! global->player[j].playerid)  continue;
+			if(global->player[j].game != game->gameid)  continue;
+			if(global->player[j].bankrupt)  continue;
+			if(global->player[j].jailed) continue;
+			if(global->player[j].location != i)  continue;
 
 			nb_tokens++;
 		}
@@ -857,44 +882,45 @@ void game_update_tokens()  {
 
 		for(j = 0, k = 0, xp = -1, yp = -1, pp = -1 ; j < MAX_PLAYERS ; j++)  {
 
-			if(! game->player[j].playerid)  continue;
-			if(game->player[j].bankrupt)  continue;
-			if(game->player[j].jailed) continue;
-			if(game->player[j].location != i)  continue;
+			if(! global->player[j].playerid)  continue;
+			if(global->player[j].game != game->gameid)  continue;
+			if(global->player[j].bankrupt)  continue;
+			if(global->player[j].jailed) continue;
+			if(global->player[j].location != i)  continue;
 
-			x1 = data->estate[ game->player[j].location ].x1token;
-			x2 = data->estate[ game->player[j].location ].x2token;
-			y1 = data->estate[ game->player[j].location ].y1token;
-			y2 = data->estate[ game->player[j].location ].y2token;
+			x1 = data->estate[ global->player[j].location ].x1token;
+			x2 = data->estate[ global->player[j].location ].x2token;
+			y1 = data->estate[ global->player[j].location ].y1token;
+			y2 = data->estate[ global->player[j].location ].y2token;
 
 			if(nb_tokens == 1)  {
 
-				pic_set_x(game->board_frame, game->player[j].token_pic, x1);
-				pic_set_y(game->board_frame, game->player[j].token_pic, y1);
+				pic_set_x(game->board_frame, global->player[j].token_pic, x1);
+				pic_set_y(game->board_frame, global->player[j].token_pic, y1);
 
 				break;
 			}
 
 			xs = ( ( (x2 - x1) * k) / (nb_tokens -1) ) + x1;
-			if(pp >= 0  &&  abs(xs - xp) > (data->pngfile_token_width[ game->player[pp].buffer_token ] + 2) ) {
+			if(pp >= 0  &&  abs(xs - xp) > (data->pngfile_token_width[ global->player[pp].buffer_token ] + 2) ) {
 
 				if(xs - xp > 0)
-					xs = xp + data->pngfile_token_width[ game->player[pp].buffer_token ] + 2;
+					xs = xp + data->pngfile_token_width[ global->player[pp].buffer_token ] + 2;
 				else
-					xs = xp - data->pngfile_token_width[ game->player[pp].buffer_token ] - 2;
+					xs = xp - data->pngfile_token_width[ global->player[pp].buffer_token ] - 2;
 			}
 
 			ys = ( ( (y2 - y1) * k) / (nb_tokens -1) ) + y1;
-			if(pp >= 0  &&  abs(ys - yp) > (data->pngfile_token_height[ game->player[pp].buffer_token ] + 2) ) {
+			if(pp >= 0  &&  abs(ys - yp) > (data->pngfile_token_height[ global->player[pp].buffer_token ] + 2) ) {
 
 				if(ys - yp > 0)
-					ys = yp + data->pngfile_token_height[ game->player[pp].buffer_token ] + 2;
+					ys = yp + data->pngfile_token_height[ global->player[pp].buffer_token ] + 2;
 				else
-					ys = yp - data->pngfile_token_height[ game->player[pp].buffer_token ] - 2;
+					ys = yp - data->pngfile_token_height[ global->player[pp].buffer_token ] - 2;
 			}
 
-			pic_set_x(game->board_frame, game->player[j].token_pic, xs);
-			pic_set_y(game->board_frame, game->player[j].token_pic, ys);
+			pic_set_x(game->board_frame, global->player[j].token_pic, xs);
+			pic_set_y(game->board_frame, global->player[j].token_pic, ys);
 
 			xp = xs;
 			yp = ys;
@@ -912,10 +938,11 @@ void game_update_tokens()  {
 		/* search number of jailed tokens on this estate */
 		for(j = 0, nb_tokens = 0 ; j < MAX_PLAYERS ; j++)  {
 
-			if(! game->player[j].playerid)  continue;
-			if(game->player[j].bankrupt)  continue;
-			if(! game->player[j].jailed) continue;
-			if(game->player[j].location != i)  continue;
+			if(! global->player[j].playerid)  continue;
+			if(global->player[j].game != game->gameid)  continue;
+			if(global->player[j].bankrupt)  continue;
+			if(! global->player[j].jailed) continue;
+			if(global->player[j].location != i)  continue;
 
 			nb_tokens++;
 		}
@@ -925,44 +952,45 @@ void game_update_tokens()  {
 
 		for(j = 0, k = 0, xp = -1, yp = -1, pp = -1 ; j < MAX_PLAYERS ; j++)  {
 
-			if(! game->player[j].playerid)  continue;
-			if(game->player[j].bankrupt)  continue;
-			if(! game->player[j].jailed) continue;
-			if(game->player[j].location != i)  continue;
+			if(! global->player[j].playerid)  continue;
+			if(global->player[j].game != game->gameid)  continue;
+			if(global->player[j].bankrupt)  continue;
+			if(! global->player[j].jailed) continue;
+			if(global->player[j].location != i)  continue;
 
-			x1 = data->estate[ game->player[j].location ].x1jail;
-			x2 = data->estate[ game->player[j].location ].x2jail;
-			y1 = data->estate[ game->player[j].location ].y1jail;
-			y2 = data->estate[ game->player[j].location ].y2jail;
+			x1 = data->estate[ global->player[j].location ].x1jail;
+			x2 = data->estate[ global->player[j].location ].x2jail;
+			y1 = data->estate[ global->player[j].location ].y1jail;
+			y2 = data->estate[ global->player[j].location ].y2jail;
 
 			if(nb_tokens == 1)  {
 
-				pic_set_x(game->board_frame, game->player[j].token_pic, x1);
-				pic_set_y(game->board_frame, game->player[j].token_pic, y1);
+				pic_set_x(game->board_frame, global->player[j].token_pic, x1);
+				pic_set_y(game->board_frame, global->player[j].token_pic, y1);
 
 				break;
 			}
 
 			xs = ( ( (x2 - x1) * k) / (nb_tokens -1) ) + x1;
-			if(pp >= 0  &&  abs(xs - xp) > (data->pngfile_token_width[ game->player[pp].buffer_token ] + 2) ) {
+			if(pp >= 0  &&  abs(xs - xp) > (data->pngfile_token_width[ global->player[pp].buffer_token ] + 2) ) {
 
 				if(xs - xp > 0)
-					xs = xp + data->pngfile_token_width[ game->player[pp].buffer_token ] + 2;
+					xs = xp + data->pngfile_token_width[ global->player[pp].buffer_token ] + 2;
 				else
-					xs = xp - data->pngfile_token_width[ game->player[pp].buffer_token ] - 2;
+					xs = xp - data->pngfile_token_width[ global->player[pp].buffer_token ] - 2;
 			}
 
 			ys = ( ( (y2 - y1) * k) / (nb_tokens -1) ) + y1;
-			if(pp >= 0  &&  abs(ys - yp) > (data->pngfile_token_height[ game->player[pp].buffer_token ] + 2) ) {
+			if(pp >= 0  &&  abs(ys - yp) > (data->pngfile_token_height[ global->player[pp].buffer_token ] + 2) ) {
 
 				if(ys - yp > 0)
-					ys = yp + data->pngfile_token_height[ game->player[pp].buffer_token ] + 2;
+					ys = yp + data->pngfile_token_height[ global->player[pp].buffer_token ] + 2;
 				else
-					ys = yp - data->pngfile_token_height[ game->player[pp].buffer_token ] - 2;
+					ys = yp - data->pngfile_token_height[ global->player[pp].buffer_token ] - 2;
 			}
 
-			pic_set_x(game->board_frame, game->player[j].token_pic, xs);
-			pic_set_y(game->board_frame, game->player[j].token_pic, ys);
+			pic_set_x(game->board_frame, global->player[j].token_pic, xs);
+			pic_set_y(game->board_frame, global->player[j].token_pic, ys);
 
 			xp = xs;
 			yp = ys;
@@ -984,10 +1012,10 @@ void game_start_move_token(guint32 playerid)  {
 	pslot = get_player_slot_with_playerid(playerid);
 	if(pslot < 0)  return;
 
-	if(game->player[pslot].directmove)  {
+	if(global->player[pslot].directmove)  {
 
-		game->player[pslot].location = game->player[pslot].location_to;
-		sendstr = g_strdup_printf(".t%d\n", game->player[pslot].location);
+		global->player[pslot].location = global->player[pslot].location_to;
+		sendstr = g_strdup_printf(".t%d\n", global->player[pslot].location);
 		client_send(game->connectid, sendstr, strlen(sendstr));
 		g_free(sendstr);
 	}
@@ -1007,14 +1035,15 @@ gboolean game_move_tokens()  {
 
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(! game->player[i].playerid)  continue;
-		if(game->player[i].location == game->player[i].location_to)  continue;
+		if(! global->player[i].playerid)  continue;
+		if(global->player[i].game != game->gameid)  continue;
+		if(global->player[i].location == global->player[i].location_to)  continue;
 
-		game->player[i].location++;
-		if(game->player[i].location >= data->number_estates)
-			game->player[i].location = 0;
+		global->player[i].location++;
+		if(global->player[i].location >= data->number_estates)
+			global->player[i].location = 0;
 
-		sendstr = g_strdup_printf(".t%d\n", game->player[i].location);
+		sendstr = g_strdup_printf(".t%d\n", global->player[i].location);
 		client_send(game->connectid, sendstr, strlen(sendstr));
 		g_free(sendstr);
 
@@ -1025,3 +1054,14 @@ gboolean game_move_tokens()  {
 
 	return TRUE;
 }
+
+
+void game_delete_player(guint32 id)  {
+
+	/* delete a player */
+	if(global->player[id].name) g_free(global->player[id].name);
+	if(global->player[id].host) g_free(global->player[id].host);
+	if(global->player[id].image) g_free(global->player[id].image);
+	memset(&global->player[id], 0, sizeof(_player) );
+	game->nb_players--;
+}
