@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2003 Rochet Sylvain
+ *  Copyright (C) 2002-2004 Rochet Sylvain
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -84,7 +84,7 @@ void interface_create_menubar()  {
 
 	gtk_menu_bar_append(GTK_MENU_BAR(MenuBar), Button);
 //	gtk_signal_connect(GTK_OBJECT(Button), "activate", (GtkSignalFunc)CallBack, (gpointer)"Menu Game");
-   
+
 	Menu = gtk_menu_new();
 	gtk_menu_set_title(GTK_MENU(Menu), "Game Menu");
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(Button), Menu);
@@ -155,14 +155,14 @@ void interface_create_menubar()  {
 	gtk_menu_append(GTK_MENU(Menu), MenuItem);
 	gtk_signal_connect(GTK_OBJECT(MenuItem), "activate", (GtkSignalFunc)Callback_ShowConfiguration, NULL);
 
-	MenuItem = gtk_menu_item_new_with_label("Themes (unavailable yet)");
-	gtk_menu_append(GTK_MENU(Menu), MenuItem);
-	gtk_signal_connect(GTK_OBJECT(MenuItem), "activate", (GtkSignalFunc)Callback_menu_Themes, NULL);
+//	MenuItem = gtk_menu_item_new_with_label("Themes (unavailable yet)");
+//	gtk_menu_append(GTK_MENU(Menu), MenuItem);
+//	gtk_signal_connect(GTK_OBJECT(MenuItem), "activate", (GtkSignalFunc)Callback_menu_Themes, NULL);
 
 	/* HELP MENU */
 	Button = gtk_menu_item_new_with_label("Help");
 	gtk_menu_item_right_justify(GTK_MENU_ITEM(Button));
-   
+
 	gtk_menu_bar_append(GTK_MENU_BAR(MenuBar), Button);
 //	gtk_signal_connect(GTK_OBJECT(Button), "activate", (GtkSignalFunc)CallBack, (gpointer)"Menu Help");
 
@@ -465,8 +465,8 @@ void interface_create_gameconfigpage()  {
 	gtk_widget_set_usize(MessageFrame , 0, 130);
 	gtk_box_pack_start(GTK_BOX(VerticalBoxLeft), MessageFrame, FALSE, TRUE, 0);
 
-	gtk_text_set_point(GTK_TEXT(global->Message), 0);
-	gtk_text_forward_delete(GTK_TEXT(global->Message), gtk_text_get_length(GTK_TEXT(global->Message)) );
+//	gtk_text_set_point(GTK_TEXT(global->Message), 0);
+//	gtk_text_forward_delete(GTK_TEXT(global->Message), gtk_text_get_length(GTK_TEXT(global->Message)) );
 	global->message_nb_lines = 0;
 	interface_create_message(MessageFrame);
 
@@ -580,13 +580,13 @@ void interface_create_gameboardpage()  {
 				GtkWidget *PlayerVerticalBox;
 					GtkWidget *PlayerHorizontalBox;
 						GtkWidget *AlignementTokenPlayer;
-							//game->player[].playerlist_token_gdkrgb
+							//global->player[].playerlist_token_gdkrgb
 						GtkWidget *PlayerTable;
-							//game->player[i].playerlist_AlignementNamePlayer;
-								//game->player[].playerlist_LabelNamePlayer
-							//game->player[i].playerlist_AlignementMoneyPlayer;
-								//game->player[].playerlist_LabelMoneyPlayer
-							//game->player[].playerlist_cards_gdkrgb
+							//global->player[i].playerlist_AlignementNamePlayer;
+								//global->player[].playerlist_LabelNamePlayer
+							//global->player[i].playerlist_AlignementMoneyPlayer;
+								//global->player[].playerlist_LabelMoneyPlayer
+							//global->player[].playerlist_cards_gdkrgb
 						GtkWidget *PlayerPopupMenu;
 					GtkWidget *PlayerSeparator;
 				//game->BoxMessageAndChat
@@ -641,10 +641,11 @@ void interface_create_gameboardpage()  {
 	/* set buffer_token & buffer_star */
 	for(i = 0, j = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(! game->player[i].playerid) continue;
+		if(! global->player[i].playerid) continue;
+		if(global->player[i].game != game->gameid) continue;
 
-		game->player[i].buffer_token = j;
-		game->player[i].buffer_star = j;
+		global->player[i].buffer_token = j;
+		global->player[i].buffer_star = j;
 		j++;
 	}
 
@@ -656,9 +657,10 @@ void interface_create_gameboardpage()  {
 	/* player list game */
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(! game->player[i].playerid) continue;
+		if(! global->player[i].playerid) continue;
+		if(global->player[i].game != game->gameid) continue;
 
-		token = game->player[i].buffer_token;
+		token = global->player[i].buffer_token;
 
 		/* separator */
 		PlayerSeparator = gtk_hseparator_new();
@@ -681,7 +683,7 @@ void interface_create_gameboardpage()  {
 
 		/* building popup player menu */
 		PlayerPopupMenu = gtk_menu_new();
-		text = g_strdup_printf("%d", game->player[i].playerid);
+		text = g_strdup_printf("%d", global->player[i].playerid);
 		gtk_object_set_data_full(GTK_OBJECT(PlayerPopupMenu), "playerid", text, g_free);
 		gtk_object_set_data(GTK_OBJECT(EventBox), "menu", PlayerPopupMenu);
 
@@ -716,13 +718,13 @@ void interface_create_gameboardpage()  {
 		/* token player in playerlist */
 		AlignementTokenPlayer = gtk_alignment_new(1.0, 1.0, 0.0, 0.0);
 		gtk_box_pack_start(GTK_BOX(PlayerHorizontalBox), AlignementTokenPlayer, TRUE, TRUE, 0);
-		game->player[i].playerlist_token_gdkrgb = gtk_drawing_area_new();
-		gtk_drawing_area_size( GTK_DRAWING_AREA (game->player[i].playerlist_token_gdkrgb), data->playerlist_token_width,  data->playerlist_token_height );
-		gtk_container_add(GTK_CONTAINER(AlignementTokenPlayer), game->player[i].playerlist_token_gdkrgb);
-		gtk_signal_connect (GTK_OBJECT (game->player[i].playerlist_token_gdkrgb), "expose_event", GTK_SIGNAL_FUNC (on_playertoken_expose), NULL);
+		global->player[i].playerlist_token_gdkrgb = gtk_drawing_area_new();
+		gtk_drawing_area_size( GTK_DRAWING_AREA (global->player[i].playerlist_token_gdkrgb), data->playerlist_token_width,  data->playerlist_token_height );
+		gtk_container_add(GTK_CONTAINER(AlignementTokenPlayer), global->player[i].playerlist_token_gdkrgb);
+		gtk_signal_connect (GTK_OBJECT (global->player[i].playerlist_token_gdkrgb), "expose_event", GTK_SIGNAL_FUNC (on_playertoken_expose), NULL);
 
 		text = g_strdup_printf("%d", i);
-		gtk_object_set_data_full(GTK_OBJECT(game->player[i].playerlist_token_gdkrgb), "player", text, g_free);
+		gtk_object_set_data_full(GTK_OBJECT(global->player[i].playerlist_token_gdkrgb), "player", text, g_free);
 
 		/* name, money, cards owned by player */
 		PlayerTable = gtk_table_new(2, 2, FALSE);
@@ -731,42 +733,42 @@ void interface_create_gameboardpage()  {
 		gtk_box_pack_start(GTK_BOX(PlayerHorizontalBox), PlayerTable, FALSE, FALSE, 0);
 
 		/* cards */
-		game->player[i].playerlist_cards_gdkrgb = gtk_drawing_area_new();
-		gtk_drawing_area_size( GTK_DRAWING_AREA (game->player[i].playerlist_cards_gdkrgb), data->playerlist_cards_width,  data->playerlist_cards_height );
-		gtk_table_attach_defaults(GTK_TABLE(PlayerTable),  game->player[i].playerlist_cards_gdkrgb,  0,  2,  1,  2);
-		gtk_signal_connect (GTK_OBJECT (game->player[i].playerlist_cards_gdkrgb), "expose_event", GTK_SIGNAL_FUNC (on_playercards_expose), NULL);
+		global->player[i].playerlist_cards_gdkrgb = gtk_drawing_area_new();
+		gtk_drawing_area_size( GTK_DRAWING_AREA (global->player[i].playerlist_cards_gdkrgb), data->playerlist_cards_width,  data->playerlist_cards_height );
+		gtk_table_attach_defaults(GTK_TABLE(PlayerTable),  global->player[i].playerlist_cards_gdkrgb,  0,  2,  1,  2);
+		gtk_signal_connect (GTK_OBJECT (global->player[i].playerlist_cards_gdkrgb), "expose_event", GTK_SIGNAL_FUNC (on_playercards_expose), NULL);
 
 		text = g_strdup_printf("%d", i);
-		gtk_object_set_data_full(GTK_OBJECT(game->player[i].playerlist_cards_gdkrgb), "player", text, g_free);
+		gtk_object_set_data_full(GTK_OBJECT(global->player[i].playerlist_cards_gdkrgb), "player", text, g_free);
 
 		/* name */
-		game->player[i].playerlist_AlignementNamePlayer = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-		gtk_table_attach_defaults(GTK_TABLE(PlayerTable),  game->player[i].playerlist_AlignementNamePlayer,  0,  1,  0,  1);
-		game->player[i].playerlist_LabelNamePlayer = gtk_label_new(game->player[i].name);
-		gtk_object_set_data(GTK_OBJECT(game->player[i].playerlist_AlignementNamePlayer), "namelabel", game->player[i].playerlist_LabelNamePlayer);
-		gtk_container_add(GTK_CONTAINER(game->player[i].playerlist_AlignementNamePlayer), game->player[i].playerlist_LabelNamePlayer);
+		global->player[i].playerlist_AlignementNamePlayer = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
+		gtk_table_attach_defaults(GTK_TABLE(PlayerTable),  global->player[i].playerlist_AlignementNamePlayer,  0,  1,  0,  1);
+		global->player[i].playerlist_LabelNamePlayer = gtk_label_new(global->player[i].name);
+		gtk_object_set_data(GTK_OBJECT(global->player[i].playerlist_AlignementNamePlayer), "namelabel", global->player[i].playerlist_LabelNamePlayer);
+		gtk_container_add(GTK_CONTAINER(global->player[i].playerlist_AlignementNamePlayer), global->player[i].playerlist_LabelNamePlayer);
 
-		if(game->player[i].hasturn)
-			gtk_widget_set_style(game->player[i].playerlist_LabelNamePlayer, global->style_player_turn);
+		if(global->player[i].hasturn)
+			gtk_widget_set_style(global->player[i].playerlist_LabelNamePlayer, global->style_player_turn);
 		else
-			gtk_widget_set_style(game->player[i].playerlist_LabelNamePlayer, global->style_player_normal);
+			gtk_widget_set_style(global->player[i].playerlist_LabelNamePlayer, global->style_player_normal);
 
 		/* money */
-		game->player[i].playerlist_AlignementMoneyPlayer = gtk_alignment_new(1.0, 0.0, 0.0, 0.0);
-		gtk_table_attach_defaults(GTK_TABLE(PlayerTable),  game->player[i].playerlist_AlignementMoneyPlayer,  1,  2,  0,  1);
-		text = g_strdup_printf("%d", game->player[i].money);
-		game->player[i].playerlist_LabelMoneyPlayer = gtk_label_new(text);
+		global->player[i].playerlist_AlignementMoneyPlayer = gtk_alignment_new(1.0, 0.0, 0.0, 0.0);
+		gtk_table_attach_defaults(GTK_TABLE(PlayerTable),  global->player[i].playerlist_AlignementMoneyPlayer,  1,  2,  0,  1);
+		text = g_strdup_printf("%d", global->player[i].money);
+		global->player[i].playerlist_LabelMoneyPlayer = gtk_label_new(text);
 		g_free(text);
-		gtk_object_set_data(GTK_OBJECT(game->player[i].playerlist_AlignementMoneyPlayer), "moneylabel", game->player[i].playerlist_LabelMoneyPlayer);
-		gtk_container_add(GTK_CONTAINER(game->player[i].playerlist_AlignementMoneyPlayer), game->player[i].playerlist_LabelMoneyPlayer);
+		gtk_object_set_data(GTK_OBJECT(global->player[i].playerlist_AlignementMoneyPlayer), "moneylabel", global->player[i].playerlist_LabelMoneyPlayer);
+		gtk_container_add(GTK_CONTAINER(global->player[i].playerlist_AlignementMoneyPlayer), global->player[i].playerlist_LabelMoneyPlayer);
 
-		if(game->player[i].hasturn)
-			gtk_widget_set_style(game->player[i].playerlist_LabelMoneyPlayer, global->style_player_turn);
+		if(global->player[i].hasturn)
+			gtk_widget_set_style(global->player[i].playerlist_LabelMoneyPlayer, global->style_player_turn);
 		else
-			gtk_widget_set_style(game->player[i].playerlist_LabelMoneyPlayer, global->style_player_normal);
+			gtk_widget_set_style(global->player[i].playerlist_LabelMoneyPlayer, global->style_player_normal);
 
 		/* playerlist token frame */
-		frame = game->player[i].playerlist_token_frame = frame_create();
+		frame = global->player[i].playerlist_token_frame = frame_create();
 		frame_set_width(frame, data->playerlist_token_width);
 		frame_set_height(frame, data->playerlist_token_height);
 		if(data->playerlist_token_bgcolor < 0)
@@ -775,24 +777,24 @@ void interface_create_gameboardpage()  {
 			frame_set_bgcolor(frame, data->playerlist_token_bgcolor);
 
 		/* playerlist token alpha */
-		pic = game->player[i].playerlist_token_alpha = pic_create(frame);
+		pic = global->player[i].playerlist_token_alpha = pic_create(frame);
 		pic_is_alphamask(frame, pic);
 		pic_set_width(frame, pic, data->pngfile_token_width[token]);
 		pic_set_height(frame, pic, data->pngfile_token_height[token]);
 		pic_buff(frame, pic, data->pngfile_token_alpha[token]);
 
 		/* playerlist token pic */
-		pic = game->player[i].playerlist_token_pic = pic_create(frame);
+		pic = global->player[i].playerlist_token_pic = pic_create(frame);
 		pic_set_x(frame, pic, (data->playerlist_token_width - data->pngfile_token_width[token]) / 2);
 		pic_set_y(frame, pic, (data->playerlist_token_height - data->pngfile_token_height[token]) / 2);
 		pic_set_z(frame, pic, 0);
 		pic_set_width(frame, pic, data->pngfile_token_width[token]);
 		pic_set_height(frame, pic, data->pngfile_token_height[token]);
-		pic_set_alphamask(frame, pic, game->player[i].playerlist_token_alpha);
+		pic_set_alphamask(frame, pic, global->player[i].playerlist_token_alpha);
 		pic_buff(frame, pic, data->pngfile_token_image[token]);
 
 		/* engine frame cards */
-		frame = game->player[i].playerlist_cards_frame = frame_create();
+		frame = global->player[i].playerlist_cards_frame = frame_create();
 		frame_set_width(frame, data->playerlist_cards_width);
 		frame_set_height(frame, data->playerlist_cards_height);
 		if(data->playerlist_cards_bgcolor < 0)
@@ -803,7 +805,7 @@ void interface_create_gameboardpage()  {
 		/* cards */
 		for(j = 0 ; j < data->number_playerlist_card ; j++)  {
 
-			pic = game->player[i].playerlist_cards_pic[j] = pic_create(frame);
+			pic = global->player[i].playerlist_cards_pic[j] = pic_create(frame);
 			pic_set_x(frame, pic, data->playerlist_card[j].x);
 			pic_set_y(frame, pic, data->playerlist_card[j].y);
 			pic_set_z(frame, pic, data->playerlist_card[j].z);
@@ -811,7 +813,7 @@ void interface_create_gameboardpage()  {
 			pic_set_width(frame, pic, data->pngfile_card_width[t]);
 			pic_set_height(frame, pic, data->pngfile_card_height[t]);
 
-			if(game->estate[ data->playerlist_card[j].estateid ].owner  ==  game->player[i].playerid)  {
+			if(game->estate[ data->playerlist_card[j].estateid ].owner  ==  global->player[i].playerid)  {
 
 				if(game->estate[ data->playerlist_card[j].estateid ].mortgaged)  {
 
@@ -1101,7 +1103,7 @@ void interface_create_gameboardpage()  {
 		/* if estate owned  &&  not mortgaged */
 		if(game->estate[i].owner > 0  &&  !game->estate[i].mortgaged)  {
 
-			star = game->player[ get_player_slot_with_playerid( game->estate[i].owner ) ].buffer_star;
+			star = global->player[ get_player_slot_with_playerid( game->estate[i].owner ) ].buffer_star;
 
 			pic = game->estate[i].star_alpha;
 			pic_set_width(frame, pic, data->pngfile_star_width[star]);
@@ -1118,7 +1120,7 @@ void interface_create_gameboardpage()  {
 		/* if estate owned  &&  mortgaged */
 		else  if(game->estate[i].owner > 0  &&  game->estate[i].mortgaged)  {
 
-			star = game->player[ get_player_slot_with_playerid( game->estate[i].owner ) ].buffer_star;
+			star = global->player[ get_player_slot_with_playerid( game->estate[i].owner ) ].buffer_star;
 
 			pic = game->estate[i].star_alpha;
 			pic_set_width(frame, pic, data->pngfile_star_m_width[star]);
@@ -1189,25 +1191,26 @@ void interface_create_gameboardpage()  {
 	/* pic for tokens */
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(! game->player[i].playerid) continue;
+		if(! global->player[i].playerid) continue;
+		if(global->player[i].game != game->gameid)  continue;
 
-		token = game->player[i].buffer_token;
+		token = global->player[i].buffer_token;
 
 		/* token alpha */
-		pic = game->player[i].token_alpha = pic_create(frame);
+		pic = global->player[i].token_alpha = pic_create(frame);
 		pic_is_alphamask(frame, pic);
 		pic_set_width(frame, pic, data->pngfile_token_width[token]);
 		pic_set_height(frame, pic, data->pngfile_token_height[token]);
 		pic_buff(frame, pic, data->pngfile_token_alpha[token]);
 
 		/* token */
-		pic = game->player[i].token_pic = pic_create(frame);
+		pic = global->player[i].token_pic = pic_create(frame);
 		pic_set_z(frame, pic, 1);
 		pic_set_width(frame, pic, data->pngfile_token_width[token]);
 		pic_set_height(frame, pic, data->pngfile_token_height[token]);
-		pic_set_alphamask(frame, pic, game->player[i].token_alpha);
+		pic_set_alphamask(frame, pic, global->player[i].token_alpha);
 		pic_buff(frame, pic, data->pngfile_token_image[token]);
-		if(game->player[i].bankrupt)
+		if(global->player[i].bankrupt)
 			pic_unshow(frame, pic);
 
 		game_update_tokens();
@@ -1216,7 +1219,7 @@ void interface_create_gameboardpage()  {
 
 	gtk_widget_show_all(global->MainVerticalBox);
 
-	if(game->player[ get_player_slot_with_playerid(game->my_playerid) ].can_roll)
+	if(global->player[ get_player_slot_with_playerid(game->my_playerid) ].can_roll)
 		gtk_widget_show(CmdButton);
 	else
 		gtk_widget_hide(CmdButton);
@@ -1328,6 +1331,9 @@ void interface_create_message(GtkWidget *container)  {
 		//global->Message
 		GtkWidget* MessageVerticalScrollbar;
 
+	GtkTextBuffer *textbuff;
+	GtkTextIter textiter;
+
 	/* also created, just reparent */
 	if(global->MessageBox)  {
 
@@ -1335,16 +1341,22 @@ void interface_create_message(GtkWidget *container)  {
 		return;
 	}
 
-	global->MessageBox = gtk_hbox_new(FALSE, 0);
+	global->MessageBox = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(global->MessageBox),
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(container), global->MessageBox);
 
-	global->Message = gtk_text_new(NULL, NULL);
+	/* Create the text view */
+	global->Message = gtk_text_view_new();
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(global->Message), FALSE);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(global->Message), GTK_WRAP_CHAR);
+	textbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(global->Message));
+	gtk_text_buffer_get_end_iter(textbuff, &textiter);
+	gtk_text_buffer_create_mark(textbuff, "endmark", &textiter, FALSE);
+
 	global->message_nb_lines = 0;
-	gtk_text_set_editable(GTK_TEXT(global->Message), FALSE);
-	gtk_text_set_word_wrap(GTK_TEXT(global->Message), TRUE);
-	gtk_box_pack_start(GTK_BOX(global->MessageBox), global->Message, TRUE, TRUE, 0);
-	MessageVerticalScrollbar = gtk_vscrollbar_new( GTK_TEXT(global->Message)->vadj );
-	gtk_box_pack_start(GTK_BOX(global->MessageBox), MessageVerticalScrollbar, FALSE, FALSE, 0);
+
+	gtk_container_add(GTK_CONTAINER(global->MessageBox), global->Message);
 }
 
 
@@ -1377,11 +1389,13 @@ void interface_destroy_message()  {
 void interface_create_chat(GtkWidget *container)  {
 
 	//game->ChatBox
-		GtkWidget *ChatFrameHorizontalBox;
+		GtkWidget *ChatScrolledWindow;
 			//game->Chat
-			GtkWidget* ChatVerticalScrollbar;
 		GtkWidget *SendEntry;
 
+	GtkTextBuffer *textbuff;
+	GtkTextIter textiter;
+
 	if(! game) return;
 
 	/* also created, just reparent */
@@ -1394,17 +1408,23 @@ void interface_create_chat(GtkWidget *container)  {
 	game->ChatBox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(container), game->ChatBox);
 
-	ChatFrameHorizontalBox = gtk_hbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(game->ChatBox), ChatFrameHorizontalBox);
+//	ChatFrameHorizontalBox = gtk_hbox_new(FALSE, 0);
+//	gtk_container_add(GTK_CONTAINER(game->ChatBox), ChatFrameHorizontalBox);
 
-	game->Chat = gtk_text_new(NULL, NULL);
+	game->Chat = gtk_text_view_new();
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(game->Chat), FALSE);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(game->Chat), GTK_WRAP_CHAR);
+	textbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(game->Chat));
+	gtk_text_buffer_get_end_iter(textbuff, &textiter);
+	gtk_text_buffer_create_mark(textbuff, "endmark", &textiter, FALSE);
 	game->chat_nb_lines = 0;
-	gtk_text_set_editable(GTK_TEXT(game->Chat), FALSE);
-	gtk_text_set_word_wrap(GTK_TEXT(game->Chat), TRUE);
-	gtk_box_pack_start(GTK_BOX(ChatFrameHorizontalBox), game->Chat, TRUE, TRUE, 0);
 
-	ChatVerticalScrollbar = gtk_vscrollbar_new( GTK_TEXT(game->Chat)->vadj );
-	gtk_box_pack_start(GTK_BOX(ChatFrameHorizontalBox), ChatVerticalScrollbar, FALSE, FALSE, 0);
+	/* Create a scrolled window and put the textview in it */
+	ChatScrolledWindow = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ChatScrolledWindow),
+			GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(ChatScrolledWindow), game->Chat);
+	gtk_box_pack_start(GTK_BOX(game->ChatBox), ChatScrolledWindow, TRUE, TRUE, 0);
 
 	/* send entry */
 	SendEntry = gtk_entry_new();
@@ -1444,9 +1464,10 @@ void interface_destroy_chat()  {
 void interface_create_auctionbox()  {
 
 	GtkWidget *vbox;
-		GtkWidget *hbox_text;
+		GtkWidget *scrolledwindow;
 			GtkWidget *text;
-			GtkWidget *scrollbar;
+				GtkTextBuffer *textbuff;
+				GtkTextIter textiter;
 		GtkWidget *hbox_entry;
 			GtkWidget *entry;
 			GtkWidget *Alignment;
@@ -1458,21 +1479,28 @@ void interface_create_auctionbox()  {
 	gtk_object_set_data(GTK_OBJECT(game->BoardCenter), "auctionbox", vbox);
 	gtk_box_pack_start(GTK_BOX(game->BoardCenter), vbox, TRUE, TRUE, 0);
 
-	hbox_text = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox_text, TRUE, TRUE, 0);
+//	hbox_text = gtk_hbox_new(FALSE, 0);
+//	gtk_box_pack_start(GTK_BOX(vbox), hbox_text, TRUE, TRUE, 0);
 
 	hbox_entry = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox_entry, FALSE, TRUE, 0);
 
 	/* text box */
-	text = gtk_text_new(NULL, NULL);
-	gtk_text_set_editable(GTK_TEXT(text), FALSE);
-	gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
-	gtk_object_set_data(GTK_OBJECT(game->BoardCenter), "auction_text", text);
-	gtk_box_pack_start(GTK_BOX(hbox_text), text, TRUE, TRUE, 0);
+	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
+			GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
-	scrollbar = gtk_vscrollbar_new( GTK_TEXT(text)->vadj );
-	gtk_box_pack_start(GTK_BOX(hbox_text), scrollbar, FALSE, FALSE, 0);
+	text = gtk_text_view_new();
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
+
+        textbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+        gtk_text_buffer_get_end_iter(textbuff, &textiter);
+        gtk_text_buffer_create_mark(textbuff, "endmark", &textiter, FALSE);
+
+	gtk_object_set_data(GTK_OBJECT(game->BoardCenter), "auction_text", text);
+	gtk_container_add(GTK_CONTAINER(scrolledwindow), text);
+	gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
 
 	/* send entry */
 	entry = gtk_entry_new();
@@ -1525,102 +1553,63 @@ void interface_create_auctionbox()  {
 
 
 /* create about window */
-void interface_create_aboutwin()  {
+void interface_create_aboutwin()
+{
+	GtkWidget *AboutDlg;
+	GtkWidget *Label;
+	GtkWidget *VBox;
 
-	GtkWidget *AboutWin;
-		GtkWidget *Frame;
-			GtkWidget *VBox;
-				GtkWidget *Alignment;
-					GtkWidget *Label;
-					GtkWidget *Button;
+	gchar *text;
 
-	guchar *text;
-
-	AboutWin = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_window_set_title(GTK_WINDOW(AboutWin), "GtkAtlantic: About");
-	gtk_window_set_policy(GTK_WINDOW(AboutWin), FALSE, FALSE, TRUE);
-	gtk_window_set_position(GTK_WINDOW(AboutWin), GTK_WIN_POS_CENTER);
-	gtk_window_set_transient_for(GTK_WINDOW(AboutWin), GTK_WINDOW(global->MainWin));
-	gtk_window_set_modal(GTK_WINDOW(AboutWin), TRUE);
-	gtk_widget_set_usize(AboutWin, 300, 0);
-	gtk_widget_realize(AboutWin);
-	gdk_window_set_icon(AboutWin->window, NULL, global->icon_pixmap, global->icon_mask);
-	gtk_signal_connect(GTK_OBJECT(AboutWin), "delete_event", (GtkSignalFunc)gtk_widget_destroy, NULL);
-
-	Frame = gtk_frame_new(NULL);
-	gtk_container_set_border_width(GTK_CONTAINER(Frame), 10);
-	gtk_container_add(GTK_CONTAINER(AboutWin), Frame);
+	AboutDlg = gtk_dialog_new_with_buttons("About gtkAtlantic", NULL,
+			GTK_DIALOG_MODAL, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+			NULL);
+	gtk_window_set_default_size(GTK_WINDOW(AboutDlg), 200, 400);
+	gtk_window_set_resizable(GTK_WINDOW(AboutDlg), FALSE);
+//	gdk_window_set_icon(AboutDlg->window, NULL, global->icon_pixmap,
+//			global->icon_mask);
+	g_signal_connect_swapped(G_OBJECT(AboutDlg), "response",
+			G_CALLBACK(gtk_widget_destroy), G_OBJECT(AboutDlg));
 
+	/* Create a VBox with a border */
 	VBox = gtk_vbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(Frame), VBox);
+	gtk_container_set_border_width(GTK_CONTAINER(VBox), 10);
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(AboutDlg)->vbox), VBox);
 
-	/* name  */
-	Alignment = gtk_alignment_new(0.5, 1.0, 0.0, 0.0);
-	gtk_widget_set_usize(Alignment, 0, 30);
-	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, TRUE, 0);
-	Label = gtk_label_new("GtkAtlantic");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	/* name */
+	text = g_strdup_printf("<span size=\"xx-large\"><b>GtkAtlantic %s</b></span>", VERSION);
 
-	/* version */
-	Alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, TRUE, 0);
-	text = g_strdup_printf("Version %s", VERSION);
 	Label = gtk_label_new(text);
+	gtk_label_set_use_markup(GTK_LABEL(Label), TRUE);
+	gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE,
+			FALSE, 5);
+
 	g_free(text);
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	/* web site */
-	Alignment = gtk_alignment_new(0.5, 0.0, 0.0, 0.0);
-	gtk_widget_set_usize(Alignment, 0, 30);
-	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, TRUE, 0);
 	Label = gtk_label_new("http://gtkatlantic.sourceforge.net");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE,	FALSE, 5);
 
 	/* license */
-	Alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, TRUE, 0);
-	Label = gtk_label_new("This software is distributed under");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
-
-	Alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, TRUE, 0);
-	Label = gtk_label_new("GPL Version 2 or any later version");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	Label = gtk_label_new("This software is distributed under the GPL Version 2 or any later version");
+	gtk_label_set_line_wrap(GTK_LABEL(Label), TRUE);
+	gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE,	FALSE, 10);
 
 	/* author */
-	Alignment = gtk_alignment_new(0.5, 1.0, 0.0, 0.0);
-	gtk_widget_set_usize(Alignment, 0, 30);
-	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, TRUE, 0);
-	Label = gtk_label_new("Sylvain Rochet (France)");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	Label = gtk_label_new("<b>Authors:</b>");
+	gtk_label_set_use_markup(GTK_LABEL(Label), TRUE);
+	gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE, FALSE, 0);
 
-	/* email */
-	Alignment = gtk_alignment_new(0.5, 0.0, 0.0, 0.0);
-	gtk_widget_set_usize(Alignment, 0, 30);
-	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, TRUE, 0);
-	Label = gtk_label_new("gradator@gradator.net");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	Label = gtk_label_new("Sylvain Rochet (gradator@gradator.net)");
+	gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE, FALSE, 0);
+	Label = gtk_label_new("Xavier Amado (xavier@blackbloodstudios.com)");
+        gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE, FALSE, 0);
 
 	/* IRC */
-	Alignment = gtk_alignment_new(0.5, 0.0, 0.0, 0.0);
-	gtk_widget_set_usize(Alignment, 0, 30);
-	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, TRUE, 0);
 	Label = gtk_label_new("irc.freenode.net #gtkatlantic");
-	gtk_container_add(GTK_CONTAINER(Alignment), Label);
+	gtk_box_pack_start(GTK_BOX(VBox), Label, FALSE, FALSE, 10);
 
-
-	/* ok button */
-	Alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-	gtk_widget_set_usize(Alignment, 0, 50);
-	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, FALSE, 0);
-
-	Button = gtk_button_new_with_label("OK");
-	gtk_widget_set_usize(Button, 70, 0);
-	gtk_container_add(GTK_CONTAINER(Alignment), Button);
-
-	gtk_signal_connect_object(GTK_OBJECT(Button), "clicked", (GtkSignalFunc)gtk_widget_destroy, GTK_OBJECT(AboutWin) );
-
-	gtk_widget_show_all(AboutWin);
+	gtk_widget_show_all(AboutDlg);
 }
 
 
@@ -1886,31 +1875,31 @@ void interface_create_nicknamewin()  {
 	GtkWidget *NicknameWin;
 		GtkWidget *HBox;
 			GtkWidget *Entry;
-			GtkWidget *Button;
+			GtkWidget *Label;
 
-	NicknameWin = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_window_set_title(GTK_WINDOW(NicknameWin), "GtkAtlantic: Nickname");
-	gtk_window_set_policy(GTK_WINDOW(NicknameWin), FALSE, FALSE, TRUE);
-	gtk_window_set_position(GTK_WINDOW(NicknameWin), GTK_WIN_POS_MOUSE);
-	gtk_window_set_transient_for(GTK_WINDOW(NicknameWin), GTK_WINDOW(global->MainWin));
-	gtk_widget_realize(NicknameWin);
-	gdk_window_set_icon(NicknameWin->window, NULL, global->icon_pixmap, global->icon_mask);
-	gtk_signal_connect(GTK_OBJECT(NicknameWin), "delete_event", (GtkSignalFunc)gtk_widget_destroy, NULL);
+	NicknameWin = gtk_dialog_new_with_buttons("GtkAtlantic: Nickname",
+			GTK_WINDOW(global->MainWin), GTK_DIALOG_MODAL,
+			GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+
+//	gtk_window_set_policy(GTK_WINDOW(NicknameWin), FALSE, FALSE, TRUE);
+//	gtk_widget_realize(NicknameWin);
+//	gdk_window_set_icon(NicknameWin->window, NULL, global->icon_pixmap, global->icon_mask);
+//	gtk_signal_connect(GTK_OBJECT(NicknameWin), "delete_event", (GtkSignalFunc)gtk_widget_destroy, NULL);
 
 	HBox = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(HBox), 10);
-	gtk_container_add(GTK_CONTAINER(NicknameWin), HBox);
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(NicknameWin)->vbox), HBox);
+
+	Label = gtk_label_new("Nickname:");
+	gtk_box_pack_start(GTK_BOX(HBox), Label, FALSE, FALSE, 5);
 
 	Entry = gtk_entry_new();
 	gtk_widget_set_usize(Entry, 150, 0);
 	gtk_object_set_data(GTK_OBJECT(NicknameWin), "nickname", Entry);
-	gtk_entry_set_text(GTK_ENTRY(Entry), game->player[ get_player_slot_with_playerid(game->my_playerid) ].name);	
+	gtk_entry_set_text(GTK_ENTRY(Entry), global->player[ get_player_slot_with_playerid(game->my_playerid) ].name);
 	gtk_box_pack_start(GTK_BOX(HBox), Entry, TRUE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(Entry), "activate", (GtkSignalFunc)CallBack_ok_button_changenick_window, NicknameWin);
 
-	Button = gtk_button_new_with_label("OK");
-	gtk_box_pack_start(GTK_BOX(HBox), Button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(Button), "clicked", (GtkSignalFunc)CallBack_ok_button_changenick_window, NicknameWin);
+	gtk_signal_connect(GTK_OBJECT(NicknameWin), "response", G_CALLBACK(CallBack_response_changenick_window), NicknameWin);
 
 	gtk_widget_show_all(NicknameWin);
 }
@@ -2032,7 +2021,9 @@ void interface_create_helpwin()  {
 
 	GtkWidget *HelpWin;
 		GtkWidget *Scrolled;
-			GtkWidget *Text;
+			GtkWidget *text;
+				GtkTextBuffer *textbuff;
+				GtkTextIter textiter;
 
 	FILE *file;
 	guchar *filename, *data, *complete, *tmp;
@@ -2061,10 +2052,11 @@ void interface_create_helpwin()  {
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(Scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(global->HelpWin), Scrolled);
 
-	Text = gtk_text_new(NULL, NULL);
-	gtk_text_set_editable(GTK_TEXT(Text), FALSE);
-	gtk_text_set_word_wrap(GTK_TEXT(Text), TRUE);
-	gtk_container_add(GTK_CONTAINER(Scrolled), Text);
+	/* Create the text view */
+	text = gtk_text_view_new();
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_CHAR);
+	gtk_container_add(GTK_CONTAINER(Scrolled), text);
 
 	/* put help file */
 	data = g_malloc0(1024);
@@ -2082,7 +2074,10 @@ void interface_create_helpwin()  {
 	fclose(file);
 	g_free(data);
 
-	gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, complete, strlen(complete) );
+	textbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+	gtk_text_buffer_get_end_iter(textbuff, &textiter);
+	gtk_text_buffer_insert(textbuff, &textiter, complete, strlen(complete));
+
 	g_free(complete);
 
 	gtk_widget_show_all(HelpWin);
@@ -2147,7 +2142,7 @@ void gameclist_title_showservers()  {
 /* generic window with a label into */
 void interface_create_messagewin(guchar *msg)  {
 
-	GtkWidget *MessageWin;
+/*	GtkWidget *MessageWin;
 		GtkWidget *VBox;
 			GtkWidget *Alignment;
 				GtkWidget *Label;
@@ -2166,14 +2161,14 @@ void interface_create_messagewin(guchar *msg)  {
 	VBox = gtk_vbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(VBox), 10);
 	gtk_container_add(GTK_CONTAINER(MessageWin), VBox);
-
+*/
 	/* text  */
-	Label = gtk_label_new(msg);
+/*	Label = gtk_label_new(msg);
 	gtk_label_set_line_wrap(GTK_LABEL(Label), TRUE);
 	gtk_box_pack_start(GTK_BOX(VBox), Label, TRUE, TRUE, 0);
-
+*/
 	/* ok button */
-	Alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
+/*	Alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
 	gtk_widget_set_usize(Alignment, 0, 50);
 	gtk_box_pack_start(GTK_BOX(VBox), Alignment, FALSE, FALSE, 0);
 
@@ -2184,6 +2179,7 @@ void interface_create_messagewin(guchar *msg)  {
 	gtk_signal_connect_object(GTK_OBJECT(Button), "clicked", (GtkSignalFunc)gtk_widget_destroy, GTK_OBJECT(MessageWin) );
 
 	gtk_widget_show_all(MessageWin);
+*/
 }
 
 
@@ -2201,7 +2197,7 @@ void on_playertoken_expose (GtkWidget *widget, GdkEventExpose *event)   {
 
 	guchar *ptr = gtk_object_get_data(GTK_OBJECT(widget), "player");
 
-	game->player[ atoi(ptr) ].playerlist_token_redraw = TRUE;
+	global->player[ atoi(ptr) ].playerlist_token_redraw = TRUE;
 }
 
 /* when receive event: redraw player cards */
@@ -2209,7 +2205,7 @@ void on_playercards_expose (GtkWidget *widget, GdkEventExpose *event)   {
 
 	guchar *ptr = gtk_object_get_data(GTK_OBJECT(widget), "player");
 
-	game->player[ atoi(ptr) ].playerlist_cards_redraw = TRUE;
+	global->player[ atoi(ptr) ].playerlist_cards_redraw = TRUE;
 }
 
 
@@ -2329,33 +2325,34 @@ void update_display_game()   {
 	/* draw playerlist */
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(! game->player[i].playerid) continue;
-		if(! game->player[i].playerlist_cards_gdkrgb) continue;
+		if(! global->player[i].playerid) continue;
+		if(global->player[i].game != game->gameid)  continue;
+		if(! global->player[i].playerlist_cards_gdkrgb) continue;
 
 		/* playerlist cards */
-		frameid = game->player[i].playerlist_cards_frame;
+		frameid = global->player[i].playerlist_cards_frame;
 		if(! frameid) continue;
 		eng_create_frame(frameid);
 
-		if(game->player[i].playerlist_cards_redraw  ||  eng_zone_is_modified(frameid, 0, 0, data->playerlist_cards_width -1, data->playerlist_cards_height -1) )  {
+		if(global->player[i].playerlist_cards_redraw  ||  eng_zone_is_modified(frameid, 0, 0, data->playerlist_cards_width -1, data->playerlist_cards_height -1) )  {
 
 			draw_ptr = eng_get_zone(frameid, 0, 0, data->playerlist_cards_width -1, data->playerlist_cards_height -1);
-			gdk_draw_rgb_image (game->player[i].playerlist_cards_gdkrgb->window,  game->player[i].playerlist_cards_gdkrgb->style->fg_gc[GTK_STATE_NORMAL],  0,  0,  data->playerlist_cards_width,  data->playerlist_cards_height,  GDK_RGB_DITHER_MAX,  draw_ptr,  data->playerlist_cards_width * 3);
+			gdk_draw_rgb_image (global->player[i].playerlist_cards_gdkrgb->window,  global->player[i].playerlist_cards_gdkrgb->style->fg_gc[GTK_STATE_NORMAL],  0,  0,  data->playerlist_cards_width,  data->playerlist_cards_height,  GDK_RGB_DITHER_MAX,  draw_ptr,  data->playerlist_cards_width * 3);
 
-			game->player[i].playerlist_cards_redraw = FALSE;
+			global->player[i].playerlist_cards_redraw = FALSE;
 		}
 
 
 		/* playerlist token */
-		frameid = game->player[i].playerlist_token_frame;
+		frameid = global->player[i].playerlist_token_frame;
 		eng_create_frame(frameid);
 
-		if(game->player[i].playerlist_token_redraw  ||  eng_zone_is_modified(frameid, 0, 0, data->playerlist_token_width -1, data->playerlist_token_height -1) )  {
+		if(global->player[i].playerlist_token_redraw  ||  eng_zone_is_modified(frameid, 0, 0, data->playerlist_token_width -1, data->playerlist_token_height -1) )  {
 
 			draw_ptr = eng_get_zone(frameid, 0, 0, data->playerlist_token_width -1, data->playerlist_token_height -1);
-			gdk_draw_rgb_image (game->player[i].playerlist_token_gdkrgb->window,  game->player[i].playerlist_token_gdkrgb->style->fg_gc[GTK_STATE_NORMAL],  0,  0,  data->playerlist_token_width,  data->playerlist_token_height,  GDK_RGB_DITHER_MAX,  draw_ptr,  data->playerlist_token_width * 3);
+			gdk_draw_rgb_image (global->player[i].playerlist_token_gdkrgb->window,  global->player[i].playerlist_token_gdkrgb->style->fg_gc[GTK_STATE_NORMAL],  0,  0,  data->playerlist_token_width,  data->playerlist_token_height,  GDK_RGB_DITHER_MAX,  draw_ptr,  data->playerlist_token_width * 3);
 
-			game->player[i].playerlist_token_redraw = FALSE;
+			global->player[i].playerlist_token_redraw = FALSE;
 		}
 	}
 }
