@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2003 Rochet Sylvain
+ *  Copyright (C) 2002-2004 Rochet Sylvain
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -1242,10 +1242,10 @@ void init_prototype_gtk_style()  {
 	global->style_player_turn->fg[0].blue  = 0x0000;
 
 	global->style_estate_title = gtk_style_copy (rcstyle);
-	global->style_estate_title->font = gdk_font_load("-*-times-*-r-*-*-24-*-*-*-*-*-*-*");
-	if(! global->style_estate_title->font)
+//	global->style_estate_title->font = gdk_font_load("-*-times-*-r-*-*-24-*-*-*-*-*-*-*");
+/*	if(! global->style_estate_title->font)
 		global->style_estate_title = 0;
-
+*/
 	gtk_style_unref (rcstyle);
 }
 
